/*
 *-------------------------------------------------------------------
 *
 *   TPU -- Time Processor Unit
 *
 * The TPU is an intelligent, semi-autonomous IMB module
 * designed for timing control. Operating simultaneously with
 * the CPU, the TPU executes microinstructions (microcode) from
 * the TPU control ROM. Functions are micocode programs that
 * typically schedule tasks and perform input and output
 * operations.
 *
 * For more information, refer to Motorola's "Modular Microcontroller
 * Family Time Processor Unit Reference Manual" (Motorola document
 * TPURM/AD). 
 *
 * This file has been created by John S. Gwynne for support of 
 * Motorola's 68332 MCU in the efi332 project.
 * 
 * Redistribution and use in source and binary forms are permitted
 * provided that the following conditions are met:
 * 1. Redistribution of source code and documentation must retain
 *    the above authorship, this list of conditions and the
 *    following disclaimer.
 * 2. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * This software is provided "AS IS" without warranty of any kind,
 * either expressed or implied, including, but not limited to, the
 * implied warranties of merchantability, title and fitness for a
 * particular purpose.
 *
 *------------------------------------------------------------------
 */
#ifndef _SIM_H_
#define _SIM_H_


#include <EFI.h>


/* SAM-- shift and mask */
#undef  SAM
#define SAM(a,b,c) ((a << b) & c)



/* TPU_CRB (TPU Control Register Block) base address of the TPU
   control registers */
#if SIM_MM == 0
#define TPU_CRB 0x7ffe00
#else
#undef SIM_MM
#define SIM_MM 1
#define TPU_CRB 0xfffe00
#endif 



#define TRAMMCR (volatile unsigned short int * const)(0x00 + TPU_CRB - 0x300)
				/* TRAM Module Configuration Register */
#define    TRAM_STOP  0x8000	/*    Stop Control */
#define    TRAM_RASP  0x0100	/*    TRAM Array Space Field */

#define TRAMTST (volatile unsigned short int * const)(0x02 + TPU_CRB - 0x300)
				/*  TRAM Test Register */
/* Used for factory Testing */

#define TRAMBAR (volatile unsigned short int * const)(0x04 + TPU_CRB -0x300)
				/* TRAM Base Address and Status Register */
#define    TRAM_ADDR  0xfff8	/*    TRAM Array Base Address */
#define    TRAM_RAMDS 0x1	/*    TRAM Array Disable */

#define TPUMCR (volatile unsigned short int * const)(0x00 + TPU_CRB)
				/* Module Configuration Register */
#define    TPU_STOP   0x8000	/*    Stop Bit */
#define    TPU_TCR1P  0x6000	/*    Timer Count Register 1 Prescaler */ 
#define    TPU_TCR2P  0x1800	/*    Timer Count Register 2 Prescaler */
#define    TPU_EMU    0x0400	/*    Emulation Control */
#define    TPU_T2CG   0x0200	/*    TCR2 Clock/Gate Control */
#define    TPU_STF    0x0100    /*    Stop Flag */
#define    TPU_SUPV   0x0080	/*    Supervisor Data Space */
#define    TPU_PSCK   0x0040	/*    Prescaler Clock */
#define    TPU_IARB   0x000f	/*    Interrupt Arbitration Number */

#define TPUTCR (volatile unsigned short int * const)(0x02 + TPU_CRB)
				/* TPU Test Configuration Register */
/* Used only for factor testing */

#define DSCR (volatile unsigned short int * const)(0x04 + TPU_CRB)
				/* Development support Control Register */

#define DSSR (volatile unsigned short int * const)(0x06 + TPU_CRB)
				/* Development support status Register */

#define TICR (volatile unsigned short int * const)(0x08 + TPU_CRB)
				/* Interrupt Configuration Register */
#define    TPU_CIRL   0x0700	/*    Channel Interrupt Request Level */
#define    TPU_CIBV   0x00f0	/*    Channel Interrupt Base Vector */

#define CIER (volatile unsigned short int * const)(0x0a + TPU_CRB)
				/* Channel Interrupt Enable Register */

#define CFSR0 (volatile unsigned short int * const)(0x0c + TPU_CRB)
				/* Channel Function Select Register 0 */

#define CFSR1 (volatile unsigned short int * const)(0x0e + TPU_CRB)
				/* Channel Function Select Register 1 */

#define CFSR2 (volatile unsigned short int * const)(0x10 + TPU_CRB)
				/* Channel Function Select Register 2 */

#define CFSR3 (volatile unsigned short int * const)(0x12 + TPU_CRB)
				/* Channel Function Select Register 3 */

#define HSQR0 (volatile unsigned short int * const)(0x14 + TPU_CRB)
				/* Host Service Register 0 */

#define HSQR1 (volatile unsigned short int * const)(0x16 + TPU_CRB)
				/* Host Service Register 1 */

#define HSRR0 (volatile unsigned short int * const)(0x18 + TPU_CRB)
				/* Host Service Request Register 0 */

#define HSRR1 (volatile unsigned short int * const)(0x1a + TPU_CRB)
				/* Host Service Request Register 1 */

#define CPR0 (volatile unsigned short int * const)(0x1c + TPU_CRB)
				/* Channel Priority Register 0 */

#define CPR1 (volatile unsigned short int * const)(0x1e + TPU_CRB)
				/* Channel Priority Register 1 */
#define    TPU_Disable 0x0	/*    Service Priority Disable */
#define    TPU_Low     0x1	/*    Service Priority Low */
#define    TPU_Middle  0x2	/*    Service Priority Middle */
#define    TPU_High    0x3	/*    Service Priority High */

#define CISR (volatile unsigned short int * const)(0x20 + TPU_CRB)
				/* Channel Interrupt Status Register */

#define TPU_PR0 (volatile unsigned short int * const)(0x100 + TPU_CRB)
				/* Channel 0 Parameter Registers */

#define TPU_PR1 (volatile unsigned short int * const)(0x110 + TPU_CRB)
				/* Channel 1 Parameter Registers */

#define TPU_PR2 (volatile unsigned short int * const)(0x120 + TPU_CRB)
				/* Channel 2 Parameter Registers */

#define TPU_PR3 (volatile unsigned short int * const)(0x130 + TPU_CRB)
				/* Channel 3 Parameter Registers */

#define TPU_PR4 (volatile unsigned short int * const)(0x140 + TPU_CRB)
				/* Channel 4 Parameter Registers */

#define TPU_PR5 (volatile unsigned short int * const)(0x150 + TPU_CRB)
				/* Channel 5 Parameter Registers */

#define TPU_PR6 (volatile unsigned short int * const)(0x160 + TPU_CRB)
				/* Channel 6 Parameter Registers */

#define TPU_PR7 (volatile unsigned short int * const)(0x170 + TPU_CRB)
				/* Channel 7 Parameter Registers */

#define TPU_PR8 (volatile unsigned short int * const)(0x180 + TPU_CRB)
				/* Channel 8 Parameter Registers */

#define TPU_PR9 (volatile unsigned short int * const)(0x190 + TPU_CRB)
				/* Channel 9 Parameter Registers */

#define TPU_PR10 (volatile unsigned short int * const)(0x1a0 + TPU_CRB)
				/* Channel 10 Parameter Registers */

#define TPU_PR11 (volatile unsigned short int * const)(0x1b0 + TPU_CRB)
				/* Channel 11 Parameter Registers */

#define TPU_PR12 (volatile unsigned short int * const)(0x1c0 + TPU_CRB)
				/* Channel 12 Parameter Registers */

#define TPU_PR13 (volatile unsigned short int * const)(0x1d0 + TPU_CRB)
				/* Channel 13 Parameter Registers */

#define TPU_PR14 (volatile unsigned short int * const)(0x1e0 + TPU_CRB)
				/* Channel 14 Parameter Registers */

#define TPU_PR15 (volatile unsigned short int * const)(0x1f0 + TPU_CRB)
				/* Channel 15 Parameter Registers */

#endif /* _TPU_h_ */
